// AT90S8535
#include <avr/io.h>       	//     ,   FLASH
#include <avr/interrupt.h>	//     sei()
#include <avr/signal.h>   	//     INTERRUPT
#include <avr/delay.h>    	//     _delay_loop_2
#include <string.h>       	//     sprintf

		//         B
		//      -:
		// PB0 - E; PB1 - RS; PB - RD
#define RD_1 PORTB |= 4 //PORTB = PORTB | 0b100 - RD = 1
#define RS_1 PORTB |= 2 //PORTB = PORTB | 0b010 - RS = 1
#define E_1  PORTB |= 1 //PORTB = PORTB | 0b001 - E = 1
#define RD_0 PORTB &= 3 //PORTB = PORTB & 0b011 - RD = 0
#define RS_0 PORTB &= 5 //PORTB = PORTB & 0b101 - Rs = 0
#define E_0  PORTB &= 6 //PORTB = PORTB & 0b110 - E = 0

		//       -,
		//      4-  
const unsigned char addLUT[4] = {0x80, 0xC0, 0x94, 0xD4};
unsigned char LCD_Address, LCD_Line;  //  
										//      -
unsigned int PreviousTime; //          ICP
char buffer[15]; // <eath   ,   

//    - 
void WriteNibble(unsigned char data)
{
  RD_0;						//  
  E_1;						//  
  PORTC = (data & 0x0F);	//  
  E_0;						//  
  RD_1;						//  
  _delay_loop_2(3000);		//  3,  - 
							//    
}

//    - 
void WriteByte(unsigned char data)
{
  RD_0;						//  
  E_1;						//  
  PORTC = (data >> 4);		//   
  E_0;						//  
  E_1;						//  
  PORTC = (data & 0xF);		//   
  E_0;						//  
  RD_1;						//  
							//    
}

//        LineNum
void GoToLine(char LineNum)
{
  RS_0;//  
  LCD_Address = addLUT[LineNum-1];//   
  WriteByte(LCD_Address);//     -
  RS_1;//   
  LCD_Address = 0;//  
  LCD_Line = LineNum;//   
}

/*    (   , 
   ,    
) */
void ClearLCD(void)
{
  RS_0;					//  
  WriteByte(0x01);		//  ,  
					    //   
  _delay_loop_2(10000);//  10 ,  - 
					   //     
  RS_1;				   //   
  GoToLine(1);		   //    1
}

//    col   row 
void SetLCDPosition(char row, char col)
{
  RS_0;								 //  
  LCD_Address = addLUT[row-1] + col;//   
  WriteByte(LCD_Address);			//   - 
  RS_1;								//   
  LCD_Line = row;					//   
}

//        
void ShowChar(unsigned char c)
{
  RS_1;					//  
  WriteByte(c);			//  
  LCD_Address++;		//    1
  switch (LCD_Address)	// ,   
  {						//    
    case 20: GoToLine(2); break;
    case 40: GoToLine(3); break;
	case 60: GoToLine(4); break;
	case 80: GoToLine(1); break;
  }
}

//     
void ShowStr(unsigned char *s)
{
  while (*s != 0) ShowChar(*s++);
}

//   -
void InitLCD(void)
{
  RD_1;					//   
  E_0;					
  RS_0;					//  
  _delay_loop_2(50000);	//   50 ,  -
						//     
  WriteNibble(0x33);	//  ,
  WriteNibble(0x33);	//  -   
  WriteNibble(0x33);	//  4-  
  WriteNibble(0x22);
  WriteByte(0x28);		//    ( 5  7)
  WriteByte(0x01);		
  WriteByte(0x10);		//    
  WriteByte(0x06);		//     
  WriteByte(0x0C);		//  ,  
  for (char i=0x40; i<0x5F; i++)// B  
	_delay_loop_2(10000);
	RS_0;
	WriteByte(i);
	_delay_loop_2(10000);
	ShowChar(0);
	
  }						//     
  RS_1;					//     
  SetLCDPosition(2, 4); //   "Velosity is"
  buffer[0] = 'V';
  buffer[1] = 'e';
  buffer[2] = 'l';
  buffer[3] = 'o';
  buffer[4] = 's';
  buffer[5] = 'i';
  buffer[6] = 't';
  buffer[7] = 'y';
  buffer[8] = ' ';
  buffer[9] = 'i';
  buffer[10] = 's';
  buffer[11] = '\0';
  ShowStr(buffer);		//  "Velosity is"  
}

ISR(TIMER1_CAPT_vect)			//      ICP
{
  unsigned int CurrentTime, T;			//      			
  CurrentTime = (256 * ICR1H) + ICR1L;	// ,    .
					//  ,        
  if (CurrentTime > PreviousTime) T = CurrentTime - PreviousTime;
				//   ,      
  else T = 0xFFFF - CurrentTime + PreviousTime;
				//    buffer  , 
				//      (prm - /.)
  sprintf(buffer, "%06u rpm", (unsigned long)60E6 / (unsigned long)T );
  SetLCDPosition(3, 4); 				//    
  ShowStr(buffer);						//  
  PreviousTime = CurrentTime;			//    ""
}

void main (void)
{
  DDRC = 0x0F;			//  0-4   -  ( )
  DDRB = 0x07;			//  0-3   -  ( )
  DDRD = 0;		
  PORTD = _BV(PD6);		//  PD6 -    ICP
  TIMSK = _BV(TICIE1);	//      
  TCCR1A = 0;			//    T/C1 
  TCCR1B = _BV(CS11);	//      = 8
						//    
  TCNT1 = 0;			//    0
  InitLCD();			//  	-
  sei ();				//   
  while(1);			//        ICP
}
